@@ -1,6 +1,6 @@
 package App::Info::HTTPD::Apache;
 
-# $Id: Apache.pm,v 1.38 2002/08/08 19:19:18 david Exp $
+# $Id: Apache.pm 969 2004-12-16 22:29:37Z theory $
 
 =head1 NAME
 
@@ -44,9 +44,11 @@ use App::Info::HTTPD;
 use App::Info::Util;
 use vars qw(@ISA $VERSION);
 @ISA = qw(App::Info::HTTPD);
-$VERSION = '0.22';
+$VERSION = '0.44';
+use constant WIN32 => $^O eq 'MSWin32';
 
 my $u = App::Info::Util->new;
+my @EXES = qw(ab apachectl apxs htdigest htpasswd logresolve rotatelogs);
 
 =head1 INTERFACE
 
@@ -59,43 +61,53 @@ my $u = App::Info::Util->new;
 Returns an App::Info::HTTPD::Apache object. See L<App::Info|App::Info> for a
 complete description of argument parameters.
 
-When called, C<new()> searches the file system for the F<httpd>,
-F<apache-perl>, or F<apache> application. If found, the application (hereafer
-referred to as F<httpd>, regardless of how it was actually found to be named)
-will be called by the object methods below to gather the data necessary for
-each. If F<httpd> cannot be found, then Apache is assumed not to be installed,
-and each of the object methods will return C<undef>.
+When called, C<new()> searches the the directories returned by
+C<search_bin_dirs()> for an executable with a name returned by
+C<search_exe_names()>. If found, the executable (hereafer referred to as
+F<httpd>, regardless of how it was actually found to be named) will be called
+by the object methods below to gather the data necessary for each. If F<httpd>
+cannot be found, then Apache is assumed not to be installed, and each of the
+object methods will return C<undef>.
 
-App::Info::HTTPD::Apache searches for F<httpd> along your path, as defined by
-C<File::Spec-E<gt>path>. Failing that, it searches the following directories:
+In addition to the parameters supported by the parent classes,
+L<App::Info|App::Info> and L<App::Info::HTTPD|App::Info::HTTPD>,
+this class' C<new()> method also supports:
 
 =over 4
 
-=item /usr/local/apache/bin
+=item search_conf_names
 
-=item /usr/local/bin
+An array reference of possible names for Apache configuration files. These
+will be returned by the C<search_conf_names()> method before the default
+names, and may be used by C<conf_file()> to search for the configuration file.
 
-=item /usr/local/sbin
+=item search_conf_dirs
 
-=item /usr/bin
+An array reference of possible directories in which to search for Apache
+configuration files. These will be returned by the C<search_conf_dirs()>
+method before the default directories, and may be used by C<conf_file()> to
+search for the configuration file.
 
-=item /usr/sbin
+=back
 
-=item /bin
+As well as these parameters to specify alternate names for Apache executables
+(other than F<httpd>, which you specify via the C<search_exe_names> parameter):
 
-=item /opt/apache/bin
+=over
 
-=item /etc/httpd/bin
+=item seearch_ab_names
 
-=item /etc/apache/bin
+=item seearch_apachectl_names
 
-=item /home/httpd/bin
+=item seearch_apxs_names
 
-=item /home/apache/bin
+=item seearch_htdigest_names
 
-=item /sw/bin
+=item seearch_htpasswd_names
 
-=item /sw/sbin
+=item seearch_logresolve_names
+
+=item seearch_rotatelogs_names
 
 =back
 
@@ -123,29 +135,23 @@ sub new {
     # Construct the object.
     my $self = shift->SUPER::new(@_);
 
+    for my $exe (qw(search_conf_dirs search_conf_names), map { "search_$_\_names" } @EXES) {
+        if (exists $self->{$exe}) {
+            $self->{$exe} = [$self->{$exe}] unless ref $self->{$exe} eq 'ARRAY';
+        } else {
+            $self->{$exe} = [];
+        }
+    }
+
     # Find Apache executable.
     $self->info("Looking for Apache executable");
 
-    my @paths = ($u->path,
-      qw(/usr/local/apache/bin
-         /usr/local/bin
-         /usr/local/sbin
-         /usr/bin
-         /usr/sbin
-         /bin
-         /opt/apache/bin
-         /etc/httpd/bin
-         /etc/apache/bin
-         /home/httpd/bin
-         /home/apache/bin
-         /sw/bin
-         /sw/sbin));
-
-    my @exes = qw(httpd apache-perl apache);
+    my @paths = $self->search_bin_dirs;
+    my @exes = $self->search_exe_names;
 
     if (my $exe = $u->first_cat_exe(\@exes, @paths)) {
         # We found httpd. Confirm.
-        $self->{exe} =
+        $self->{executable} =
           $self->confirm( key      => 'executable',
                           prompt   => 'Path to your httpd executable?',
                           value    => $exe,
@@ -153,7 +159,7 @@ sub new {
                           error    => 'Not an executable');
     } else {
         # Handle an unknown value.
-        $self->{exe} =
+        $self->{executable} =
           $self->unknown( key      => 'executable',
                           prompt   => 'Path to your httpd executable?',
                           callback => sub { -x },
@@ -194,7 +200,7 @@ installed, then all of the other object methods will return empty values.
 
 =cut
 
-sub installed { return $_[0]->{exe} ? 1 : undef }
+sub installed { return $_[0]->{executable} ? 1 : undef }
 
 ##############################################################################
 
@@ -230,11 +236,11 @@ Enter a valid Apache name
 my $get_version = sub {
     my $self = shift;
     $self->{-v} = 1;
-    $self->info("Executing `$self->{exe} -v`");
-    my $version = `$self->{exe} -v`;
+    $self->info(qq{Executing `"$self->{executable}" -v`});
+    my $version = `"$self->{executable}" -v`;
     unless ($version) {
         $self->error("Failed to find Apache version data with ",
-                     "`$self->{exe} -v`");
+                     qq{`"$self->{executable}" -v`});
         return;
     }
 
@@ -253,7 +259,7 @@ my $get_version = sub {
 
 sub name {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
 
     # Load data.
     $get_version->($self) unless exists $self->{-v};
@@ -298,7 +304,7 @@ Enter a valid Apache version number
 
 sub version {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
 
     # Load data.
     $get_version->($self) unless exists $self->{-v};
@@ -362,7 +368,7 @@ my $is_int = sub { /^\d+$/ };
 
 sub major_version {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Load data.
     $get_version->($self) unless exists $self->{-v};
     # Handle an unknown value.
@@ -407,7 +413,7 @@ Enter a valid Apache minor version number
 
 sub minor_version {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Load data.
     $get_version->($self) unless exists $self->{-v};
     # Handle an unknown value.
@@ -451,7 +457,7 @@ Enter a valid Apache patch version number
 
 sub patch_version {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Load data.
     $get_version->($self) unless exists $self->{-v};
     # Handle an unknown value.
@@ -498,12 +504,12 @@ Enter a valid HTTPD root
 my $get_compile_settings = sub {
     my $self = shift;
     $self->{-V} = 1;
-    $self->info("Executing `$self->{exe} -V`");
+    $self->info(qq{Executing `"$self->{executable}" -V`});
     # Get the compile settings.
-    my $data = `$self->{exe} -V`;
+    my $data = `"$self->{executable}" -V`;
     unless ($data) {
         $self->error("Unable to extract compile settings from ",
-                     "`$self->{exe} -V`");
+                     qq{`"$self->{executable}" -V`});
         return;
     }
 
@@ -516,6 +522,14 @@ my $get_compile_settings = sub {
             $_ =~ s/"$//;
             my ($k, $v) = split /\s*=\s*"/, $_;
             $self->{lc $k} = $v;
+            if (WIN32) {
+                if ($k eq 'SUEXEC_BIN') {
+                    $self->{lc $k} = 0;
+                } elsif ($k eq 'HTTPD_ROOT') {
+                    $self->{lc $k} =
+                      join('\\', (split /\\/, $self->{executable} )[0 .. 1]);
+                 }
+            }
         } elsif (/-D/) {
             $_ =~ s/^-D\s+//;
             $self->{lc $_} = 1;
@@ -523,7 +537,7 @@ my $get_compile_settings = sub {
     }
     # Issue a warning if no httpd root was found.
     $self->error("Cannot parse HTTPD root from ",
-                 "`$self->{exe} -V`") unless $self->{httpd_root};
+                 qq{`"$self->{executable}" -V`}) unless $self->{httpd_root};
 };
 
 # This code reference is used by httpd_root(), lib_dir(), bin_dir(), and
@@ -532,7 +546,7 @@ my $is_dir = sub { -d };
 
 sub httpd_root {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Get the compile settings.
     $get_compile_settings->($self) unless $self->{-V};
     # Handle an unknown value.
@@ -576,7 +590,7 @@ Enter a valid magic number
 
 sub magic_number {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Get the compile settings.
     $get_compile_settings->($self) unless $self->{-V};
     # Handle an unknown value.
@@ -628,7 +642,7 @@ Enter a valid option
 
 sub compile_option {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     # Get the compile settings.
     $get_compile_settings->($self) unless $self->{-V};
     # Handle an unknown value.
@@ -647,21 +661,11 @@ for the configuration file in a number of locations and under a number of
 names. First it tries to use the file specifed by the C<SERVER_CONFIG_FILE>
 compile option (as returned by a call to C<compile_option()>) -- and if it's a
 relative file name, it gets appended to the directory returned by
-C<httpd_root()>. If that file isn't found, C<conf_file()> then looks for the
-files F<httpd.conf> and F<httpd.conf.default> in the F<conf> subdirectory of
-the HTTPD root directory. Failing that, it looks for the following:
-
-=over 4
-
-=item /usr/share/doc/apache-perl/examples/httpd.conf
-
-=item /usr/share/doc/apache-perl/examples/httpd.conf.default
-
-=item /etc/httpd/httpd.conf
-
-=item /etc/httpd/httpd.conf.default
-
-=back
+C<httpd_root()>. If that file isn't found, C<conf_file()> then looks for a
+file with one of the names returned by C<search_conf_names()> in the F<conf>
+subdirectory of the HTTPD root directory. Failing that, it searches for them
+in each of the directories returned by C<search_conf_dirs()> until it finds a
+match.
 
 B<Events:>
 
@@ -685,25 +689,24 @@ Location of httpd.conf file?
 
 sub conf_file {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     unless (exists $self->{conf_file}) {
         $self->info("Searching for Apache configuration file");
         my $root = $self->httpd_root;
         my $conf = $self->compile_option('SERVER_CONFIG_FILE');
         $conf = $u->file_name_is_absolute($conf) ?
           $conf : $u->catfile($root, $conf) if $conf;
-        # Paths to search.
-        my @paths = ($conf ? ($conf) : (),
-                     $u->catfile($root, 'conf', 'httpd.conf'),
-                     $u->catfile($root, 'conf', 'httpd.conf.default'),
-                     "/usr/share/doc/apache-perl/examples/httpd.conf",
-                     "/usr/share/doc/apache-perl/examples/httpd.conf.default",
-                     "/etc/httpd/httpd.conf",
-                     "/etc/httpd/httpd.conf.default");
+        if ($conf && -f $conf) {
+            $self->{conf_file} = $conf;
+        } else {
+            # Paths to search.
+            my @confs = $self->search_conf_names;
 
-        $self->{conf_file} = $u->first_file(@paths)
-          or $self->error("No Apache config file found");
+            $self->{conf_file} = $u->first_cat_path(\@confs, $self->search_conf_dirs)
+              or $self->error("No Apache config file found");
+        }
     }
+
     # Handle an unknown value.
     $self->{conf_file} =
       $self->unknown( key      => 'conf_file',
@@ -782,7 +785,7 @@ my $parse_conf_file = sub {
 
 sub user {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $parse_conf_file->($self) unless exists $self->{user};
     # Handle an unknown value.
     $self->{user} = $self->unknown( key      => 'user',
@@ -834,7 +837,7 @@ Enter Apache user group name
 
 sub group {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $parse_conf_file->($self) unless exists $self->{group};
     # Handle an unknown value.
     $self->{group} =
@@ -887,7 +890,7 @@ Enter Apache TCP/IP port number
 
 sub port {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $parse_conf_file->($self) unless exists $self->{port};
     # Handle an unknown value.
     $self->{port} =
@@ -901,56 +904,45 @@ sub port {
 
 ##############################################################################
 
+=head3 executable
+
+  my $executable = $apache->executable;
+
+Returns the path to the Apache executable, which will be defined by one of the
+names returned by C<search_exe_names()>. The executable is searched for in
+C<new()>, so there are no events for this method.
+
+=cut
+
+sub executable { shift->{executable} }
+
+=head3 httpd
+
+  my $httpd = $apache->httpd;
+
+An alias for C<executable()>.
+
+##############################################################################
+
 =head3 bin_dir
 
   my $bin_dir = $apache->bin_dir;
 
-Returns the Apache binary directory path. App::Info::HTTPD::Apache simply
-looks for the F<bin> directory under the HTTPD root directory, as returned by
-C<httpd_root()>.
-
-B<Events:>
-
-=over 4
-
-=item info
-
-Executing `httpd -V`
-
-Searching for bin directory
-
-=item error
-
-Unable to extract compile settings from `httpd -V`
-
-Cannot parse HTTPD root from `httpd -V`
-
-Cannot find bin directory
-
-=item unknown
-
-Enter a valid HTTPD root
-
-Enter a valid Apache bin directory
-
-=back
+Returns the SQLite binary directory path. App::Info::HTTPD::Apache simply
+retreives it as the directory part of the path to the HTTPD executable.
 
 =cut
 
 sub bin_dir {
     my $self = shift;
-    return unless $self->{exe};
-    unless (exists $self->{bin_dir}) {{
-        my $root = $self->httpd_root || last; # Double braces allow this.
-        $self->info("Searching for bin directory");
-        $self->{bin_dir} = $u->first_cat_path('bin', $root)
-          or $self->error("Cannot find bin directory");
-    }}
-
-    # Handle unknown value.
-    $self->{bin_dir} = $self->unknown( key      => 'bin directory',
-                                       callback => $is_dir)
-      unless $self->{bin_dir};
+    return unless $self->{executable};
+    unless (exists $self->{bin_dir} ) {
+        my @parts = $u->splitpath($self->{executable});
+        $self->{bin_dir} = $u->catdir(
+            ($parts[0] eq '' ? () : $parts[0]),
+            $u->splitdir($parts[1])
+        );
+    }
     return $self->{bin_dir};
 }
 
@@ -994,11 +986,11 @@ Enter a valid Apache include directory
 
 sub inc_dir {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     unless (exists $self->{inc_dir}) {{
         my $root = $self->httpd_root || last; # Double braces allow this.
         $self->info("Searching for include directory");
-        $self->{inc_dir} = $u->first_cat_path(['include', 'inc',], $root)
+        $self->{inc_dir} = $u->first_dir($self->search_inc_dirs)
           or $self->error("Cannot find include directory");
     }}
     # Handle unknown value.
@@ -1048,24 +1040,24 @@ Enter a valid Apache library directory
 
 sub lib_dir {
     my $self = shift;
-    return unless $self->{exe};
-    unless (exists $self->{lib_dir}) {{
-        my $root = $self->httpd_root || last; # Double braces allow this.
-        $self->info("Searching for library directory");
-        if (my $d = $u->first_cat_path([qw(lib modules libexec)], $root)) {
-            # The usual suspects.
-            $self->{lib_dir} = $d;
-        } elsif ($u->first_dir('/usr/lib/apache/1.3')) {
-            # The Debian way.
-            $self->{lib_dir} = '/usr/lib/apache/1.3';
+    return unless $self->{executable};
+    unless (exists $self->{lib_dir}) {
+        if ($self->httpd_root) {
+            $self->info("Searching for library directory");
+            if (my $d = $u->first_dir($self->search_lib_dirs)) {
+                $self->{lib_dir} = $d;
+            } else {
+                $self->error("Cannot find library direcory");
+            }
         } else {
-            $self->error("Cannot find library direcory");
+            # Handle unknown value.
+            $self->{lib_dir} = $self->unknown(
+                key      => 'library directory',
+                callback => $is_dir
+            );
         }
-    }}
-    # Handle unknown value.
-    $self->{lib_dir} = $self->unknown( key      => 'library directory',
-                                       callback => $is_dir)
-      unless $self->{lib_dir};
+
+    }
     return $self->{lib_dir};
 }
 
@@ -1141,11 +1133,11 @@ Unable to extract needed data from `httpd -l`
 my $get_static_mods = sub {
     my $self = shift;
     $self->{static_mods} = undef;
-    $self->info("Executing `$self->{exe} -l`");
-    my $data = `$self->{exe} -l`;
+    $self->info(qq{Executing `"$self->{executable}" -l`});
+    my $data = `"$self->{executable}" -l`;
     unless ($data) {
         $self->error("Unable to extract needed data from ".
-                     "`$self->{exe} =l`");
+                     qq{`"$self->{executable}" =l`});
         return;
     }
 
@@ -1161,7 +1153,7 @@ my $get_static_mods = sub {
 
 sub static_mods {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $get_static_mods->($self) unless exists $self->{static_mods};
     return unless $self->{static_mods};
     return wantarray ? @{$self->{static_mods}} : $self->{static_mods};
@@ -1193,7 +1185,7 @@ Unable to extract needed data from `httpd -l`
 
 sub mod_so {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $get_static_mods->($self) unless exists $self->{static_mods};
     return $self->{mod_so};
 }
@@ -1224,7 +1216,7 @@ Unable to extract needed data from `httpd -l`
 
 sub mod_perl {
     my $self = shift;
-    return unless $self->{exe};
+    return unless $self->{executable};
     $get_static_mods->($self) unless exists $self->{static_mods};
     return $self->{mod_perl};
 }
@@ -1253,6 +1245,321 @@ Returns the Apache download URL.
 
 sub download_url { "http://www.apache.org/dist/httpd/" }
 
+##############################################################################
+
+=head3 search_exe_names
+
+  my @search_exe_names = $apache->search_exe_names;
+
+Returns a list of possible names for the Apache executable. The names are
+F<httpd>, F<apache-perl>, and F<apache> by default; F<.exe> is appended to
+each on Win32.
+
+=cut
+
+sub search_exe_names {
+    my $self = shift;
+    my @exes = qw(httpd apache-perl apache);
+    if (WIN32) { $_ .= ".exe" for @exes }
+    return ($self->SUPER::search_exe_names, @exes);
+}
+
+##############################################################################
+
+=head3 search_bin_dirs
+
+  my @search_bin_dirs = $apache->search_bin_dirs;
+
+Returns a list of possible directories in which to search an executable. Used
+by the C<new()> constructor to find an executable to execute and collect
+application info. The found directory will also be returned by the C<bin_dir>
+method.
+
+The list of directories by default consists of the path as defined by
+C<< File::Spec->path >>, as well as the following directories:
+
+=over 4
+
+=item /usr/local/apache/bin
+
+=item /usr/local/bin
+
+=item /usr/local/sbin
+
+=item /usr/bin
+
+=item /usr/sbin
+
+=item /bin
+
+=item /opt/apache/bin
+
+=item /etc/httpd/bin
+
+=item /etc/apache/bin
+
+=item /home/httpd/bin
+
+=item /home/apache/bin
+
+=item /sw/bin
+
+=item /sw/sbin
+
+=back
+
+=cut
+
+sub search_bin_dirs {
+    return (
+      shift->SUPER::search_bin_dirs,
+      $u->path,
+      qw(/usr/local/apache/bin
+         /usr/local/bin
+         /usr/local/sbin
+         /usr/bin
+         /usr/sbin
+         /bin
+         /opt/apache/bin
+         /etc/httpd/bin
+         /etc/apache/bin
+         /home/httpd/bin
+         /home/apache/bin
+         /sw/bin
+         /sw/sbin));
+}
+
+##############################################################################
+
+=head3 search_lib_dirs
+
+  my @search_lib_dirs = $apache->search_lib_dirs;
+
+Returns a list of possible directories in which to search for Apache
+libraries. By default, it returns this list of directories, each appended to
+the name of the directory returned by C<httpd_root()>:
+
+=over 4
+
+=item lib
+
+=item modules
+
+=item libexec
+
+=back
+
+=cut
+
+sub search_lib_dirs {
+    my $self = shift;
+    my $root = $self->httpd_root;
+    return (
+      $self->SUPER::search_lib_dirs,
+      ( $root
+        ? map { $u->catdir($root, $_) } qw(lib modules libexec)
+        : ()
+      ),
+      '/usr/lib/apache/1.3'
+    );
+}
+
+##############################################################################
+
+=head3 search_inc_dirs
+
+  my @search_inc_dirs = $apache->search_inc_dirs;
+
+Returns a list of possible directories in which to search for Apache include
+files. By default, it returns this list of directories, each appended to the
+name of the directory returned by C<httpd_root()>:
+
+=over 4
+
+=item include
+
+=item inc
+
+=back
+
+=cut
+
+sub search_inc_dirs {
+    my $self = shift;
+    my $root = $self->httpd_root;
+    return (
+      $self->SUPER::search_inc_dirs,
+      ( $root
+        ? map { $u->catdir($root, $_) } qw(include inc)
+        : ()
+      ),
+    );
+}
+
+##############################################################################
+
+=head3 search_conf_names
+
+  my @search_conf_dirs = $apache->search_conf_dirs;
+
+Returns a list of possible names for Apache configuration files. These will be
+used bye the C<conf_file()> method to search for Apache configuration files.
+By Default, the possible configuration file names are:
+
+=over 4
+
+=item F<httpd.conf>
+
+=item F<httpd.conf.default>
+
+=back
+
+=cut
+
+sub search_conf_names {
+    return (
+      @{ shift->{search_conf_names} },
+      qw(httpd.conf httpd.conf.default)
+    );
+}
+
+##############################################################################
+
+=head3 search_conf_dirs
+
+  my @search_conf_dirs = $apache->search_conf_dirs;
+
+Returns a list of directories in which the C<conf_file()> method will search
+for Apache configuration files.
+
+=over 4
+
+=item /usr/share/doc/apache-perl
+
+=item /etc/httpd
+
+=back
+
+=cut
+
+sub search_conf_dirs {
+    return (
+      @{ shift->{search_conf_dirs} },
+      qw(/usr/share/doc/apache-perl /etc/httpd)
+    );
+}
+
+##############################################################################
+
+=head2 Other Executable Methods
+
+These methods return the complete paths to their like-named executables.
+Apache comes with a fair number of them; we provide these methods to provide a
+path to a subset of them. Each method, when called, checks for an executable
+in the directory returned by C<bin_dir()>. The name of the executable must be
+one of the names returned by the corresponding C<search_*_names> method.
+
+The available executable methods are:
+
+=over
+
+=item ab
+
+=item apachectl
+
+=item apxs
+
+=item htdigest
+
+=item htpasswd
+
+=item logresolve
+
+=item rotatelogs
+
+=back
+
+And the corresponding search names methods are:
+
+=over
+
+=item seearch_ab_names
+
+=item seearch_apachectl_names
+
+=item seearch_apxs_names
+
+=item seearch_htdigest_names
+
+=item seearch_htpasswd_names
+
+=item seearch_logresolve_names
+
+=item seearch_rotatelogs_names
+
+=back
+
+B<Events:>
+
+=over 4
+
+=item info
+
+Looking for executable
+
+=item confirm
+
+Path to executable?
+
+=item unknown
+
+Path to executable?
+
+=back
+
+=cut
+
+my $find_exe = sub  {
+    my ($self, $key) = @_;
+    my $exe = $key . (WIN32 ? '.exe' : '');
+    my $meth = "search_$key\_names";
+
+    # Find executable.
+    $self->info("Looking for $key");
+
+    unless ($self->{$key}) {
+	my $bin = $self->bin_dir or return;
+        if (my $exe = $u->first_cat_exe([$self->$meth(), $exe], $bin)) {
+	    # We found it. Confirm.
+	    $self->{$key} = $self->confirm(
+		key      => $key,
+		prompt   => "Path to $key executable?",
+		value    => $exe,
+		callback => sub { -x },
+		error    => 'Not an executable'
+	    );
+	} else {
+	    # Handle an unknown value.
+	    $self->{$key} = $self->unknown(
+		key      => $key,
+		prompt   => "Path to $key executable?",
+		callback => sub { -x },
+		error    => 'Not an executable'
+	    );
+	}
+    }
+
+    return $self->{$key};
+};
+
+for my $exe (@EXES) {
+    no strict 'refs';
+    *{$exe} = sub { shift->$find_exe($exe) };
+    *{"search_$exe\_names"} = sub { @{ shift->{"search_$exe\_names"} } }
+}
+
+*httpd = \&executable;
+
 1;
 __END__
 
@@ -1271,13 +1578,13 @@ parsed from the F<httpd.conf> file or C<glob>bed from the file system.
 
 =head1 BUGS
 
-Report all bugs via the CPAN Request Tracker at
+Please send bug reports to <bug-app-info@rt.cpan.org> or file them at
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=App-Info>.
 
 =head1 AUTHOR
 
-David Wheeler <L<david@wheeler.net|"david@wheeler.net">> based on code by Sam
-Tregar <L<sam@tregar.com|"sam@tregar.com">>.
+David Wheeler <david@justatheory.com> based on code by Sam Tregar
+<sam@tregar.com>.
 
 =head1 SEE ALSO
 
@@ -1294,7 +1601,7 @@ L<http://perl.apache.org/> is the mod_perl home page.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002, David Wheeler. All Rights Reserved.
+Copyright (c) 2002-2004, David Wheeler. All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
